package com.gic.finance.service.outer.impl;

import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.finance.constant.TransferAccountApprovalStatusEnum;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.entity.TabTransferAccountsApproval;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.gic.finance.service.TransferAccountsApprovalService;
import com.gic.store.utils.CreateSerialNumberUtils;
import com.gic.store.utils.valid.ValidUtil;

/**
 * 转账审批
 * @author $user$
 * @ClassName: $ClassName$
 * @Description: $desc$
 * @date $date$ $time$
 */
@Service("transferAccountsApprovalApiService")
public class TransferAccountsApprovalApiServiceImpl implements TransferAccountsApprovalApiService{
    private static final Logger LOGGER = LoggerFactory.getLogger(TransferAccountsApprovalApiServiceImpl.class);
    private final static String SERVICE_NAME = "转账审批";
    @Autowired
    private TransferAccountsApprovalService transferAccountsApprovalService;
    @Override
    public ServiceResponse<Integer> initiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        //valid param
        ServiceResponse paramValid = ValidUtil.allCheckValidate(transferAccountsApprovalDTO, TransferAccountsApprovalDTO.InitiateApprovalValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        transferAccountsApprovalDTO.setCreateTime(new Date());
        transferAccountsApprovalDTO.setUpdateTime(new Date());
        transferAccountsApprovalDTO.setApprovalStatus(TransferAccountApprovalStatusEnum.TO_BE_APPROVED.getCode());
        //审批流水号生成
        String approvalSerialNumber = CreateSerialNumberUtils.createSerialNumber();
        LOGGER.info("转账审批-发起审批-审批流水号：{}", approvalSerialNumber);
        transferAccountsApprovalDTO.setApprovalNumber(approvalSerialNumber);

        transferAccountsApprovalService.initiateApproval(transferAccountsApprovalDTO);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getById(Integer transferApprovalId) {
        LOGGER.info(SERVICE_NAME + "-getById:{}", transferApprovalId);
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(TransferAccountsApprovalDTO.class, tab));
    }

    @Override
    public ServiceResponse<Void> agreeApproval(Integer transferApprovalId, Double accountAmount) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setAccountAmount(accountAmount);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.AGREE.getCode());
        dto.setApprovalTime(new Date());
        //todo 审批人等信息获取插入
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectApproval(Integer transferApprovalId) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.REJECT.getCode());
        dto.setApprovalTime(new Date());
        //todo 审批人等信息获取插入
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<TransferAccountsApprovalDTO>> listTransferAccountsApproval(String search, Integer approvalStatus, Integer enterpriseId, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = transferAccountsApprovalService.listTransferAccountsApproval(search, approvalStatus, enterpriseId, pageNum, pageSize);
        Page<TransferAccountsApprovalDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getDetailById(Integer transferApprovalId) {
        return null;
    }
}
